/*
* Copyright (c) 2013 Jonathan Perkin <jonathan@perkin.org.uk>
* Copyright (c) 2015-2017 Elias Karakoulakis <elias.karakoulakis@gmail.com>
*
* Permission to use, copy, modify, and distribute this software for any
* purpose with or without fee is hereby granted, provided that the above
* copyright notice and this permission notice appear in all copies.
*
* THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
* WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
* ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
* WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
* ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
* OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "openzwave.hpp"

using namespace v8;
using namespace node;

namespace OZW {
	/*
	* Generic value set.
	*/
	// =================================================================
	NAN_METHOD(OZW::SetValue)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			uint8 validx  =  (info[0]->IsObject()) ? 1 : 4;
			switch ((*vit).GetType()) {
				case OpenZWave::ValueID::ValueType_Bool: {
					bool val = Nan::To<Boolean>(info[validx]).ToLocalChecked()->Value();
					OpenZWave::Manager::Get()->SetValue(*vit, val);
					break;
				}
				case OpenZWave::ValueID::ValueType_Byte: {
					uint8 val = Nan::To<Integer>(info[validx]).ToLocalChecked()->Value();
					OpenZWave::Manager::Get()->SetValue(*vit, val);
					break;
				}
				case OpenZWave::ValueID::ValueType_Decimal: {
					float val = Nan::To<Number>(info[validx]).ToLocalChecked()->NumberValue();
					OpenZWave::Manager::Get()->SetValue(*vit, val);
					break;
				}
				case OpenZWave::ValueID::ValueType_Int: {
					int32 val = Nan::To<Integer>(info[validx]).ToLocalChecked()->Value();
					OpenZWave::Manager::Get()->SetValue(*vit, val);
					break;
				}
				case OpenZWave::ValueID::ValueType_List: {
					std::string val(*Nan::Utf8String( info[validx] ));
					OpenZWave::Manager::Get()->SetValue(*vit, val);
					break;
				}
				case OpenZWave::ValueID::ValueType_Short: {
					int16 val = Nan::To<Integer>(info[validx]).ToLocalChecked()->Value();
					OpenZWave::Manager::Get()->SetValue(*vit, val);
					break;
				}
				case OpenZWave::ValueID::ValueType_String: {
					std::string val(*Nan::Utf8String( info[validx] ));
					OpenZWave::Manager::Get()->SetValue(*vit, val);
					break;
				}
				case OpenZWave::ValueID::ValueType_Schedule: {
					Nan::ThrowTypeError("please use the specialized xxxSwitchPoint commands instead of setValue for setting thermostat schedules");
					break;
				}
				case OpenZWave::ValueID::ValueType_Button: {
					if (checkType(info[validx]->IsBoolean())) {
						if (Nan::To<Boolean>(info[validx]).ToLocalChecked()->Value()) {
							OpenZWave::Manager::Get()->PressButton(*vit);
						} else {
							OpenZWave::Manager::Get()->ReleaseButton(*vit);
						}
					}
					break;
				}
				case OpenZWave::ValueID::ValueType_Raw: {
					checkType(Buffer::HasInstance(info[validx]));
					uint8 *val = (uint8*)Buffer::Data(info[validx]);
					uint8 len  = Buffer::Length(info[validx]);
					OpenZWave::Manager::Get()->SetValue(*vit, val, len);
					break;
				}
			}
		}
	}

	// =================================================================
	NAN_METHOD(OZW::SetValueLabel)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueid, label");
		OpenZWave::ValueID* vit = populateValueId(info);
		uint8 validx  =  (info[0]->IsObject()) ? 1 : 4;
		std::string label(*Nan::Utf8String( info[validx] ));

		if (vit) {
			OpenZWave::Manager::Get()->SetValueLabel(*vit, label);
		}
	}

	/*
	* Refresh value from Z-Wave network
	*/
	// =================================================================
	NAN_METHOD(OZW::RefreshValue)
			// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			const bool ok = OpenZWave::Manager::Get()->RefreshValue(*vit);
			info.GetReturnValue().Set(Nan::New<Boolean>(ok));
		}
	}

	/*
	* Ask OZW to verify the value change before notifying the application
	*/
	// =================================================================
	NAN_METHOD(OZW::SetChangeVerified)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			const uint8 validx = (info[0]->IsObject()) ? 1 : 4;
			if (checkType(info[validx]->IsBoolean())) {
				bool b = Nan::To<Boolean>(info[validx]).ToLocalChecked()->Value();
				OpenZWave::Manager::Get()->SetChangeVerified(*vit, b);
			}
		}
	}

	/*
	* Get number of thermostat switch points
	*/
	// =================================================================
	NAN_METHOD(OZW::GetNumSwitchPoints)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			if ((*vit).GetType() != OpenZWave::ValueID::ValueType_Schedule)  {
				Nan::ThrowTypeError("OpenZWave valueId is not a ValueType_Schedule");
			}
			info.GetReturnValue().Set(Nan::New<Integer>(
				OpenZWave::Manager::Get()->GetNumSwitchPoints(*vit)
			));
		}
	}

	// =================================================================
	NAN_METHOD(OZW::GetSwitchPoint)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		uint8 idx, o_hours, o_minutes;
		int8  o_setback;

		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			if ((*vit).GetType() != OpenZWave::ValueID::ValueType_Schedule ) {
				Nan::ThrowTypeError("OpenZWave valueId is not a ValueType_Schedule");
			}
			uint8 idxpos  =  (info[0]->IsObject()) ? 1 : 4;
			if ((info.Length() < idxpos) || !info[idxpos]->IsNumber()) {
				Nan::ThrowTypeError("must supply an integer index after the valueId");
			} else {
				idx = Nan::To<Number>(info[idxpos]).ToLocalChecked()->Value();
				OpenZWave::Manager::Get()->GetSwitchPoint(*vit, idx, &o_hours, &o_minutes, &o_setback);
				Local<Object> o  = Nan::New<Object>();
				Nan::Set(o,
					Nan::New<String>("hours").ToLocalChecked(),
					Nan::New<Integer>(o_hours));
				Nan::Set(o,
					Nan::New<String>("minutes").ToLocalChecked(),
					Nan::New<Integer>(o_minutes));
				Nan::Set(o,
					Nan::New<String>("setback").ToLocalChecked(),
					Nan::New<Integer>(o_setback));
				info.GetReturnValue().Set(o);
			}
		}
	}

	// Clears all switch points from the schedule.
	// =================================================================
	NAN_METHOD(OZW::ClearSwitchPoints)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			if ((*vit).GetType() != OpenZWave::ValueID::ValueType_Schedule ) {
				Nan::ThrowTypeError("OpenZWave valueId is not a ValueType_Schedule");
			} else {
				OpenZWave::Manager::Get()->ClearSwitchPoints(*vit);
			}
		}
	}

	// =================================================================
	NAN_METHOD(OZW::SetSwitchPoint)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			if ((*vit).GetType() != OpenZWave::ValueID::ValueType_Schedule ) {
				Nan::ThrowTypeError("OpenZWave valueId is not a ValueType_Schedule");
			} else {
				uint8 idxpos  =  (info[0]->IsObject()) ? 1 : 4;
				if ((info.Length() < idxpos) || !info[idxpos]->IsObject()) {
					Nan::ThrowTypeError("must supply a switchpoint object");
				} else {
					Local<Object> sp = info[idxpos]->ToObject();
					OpenZWave::Manager::Get()->SetSwitchPoint(*vit,
						Nan::To<Number>(Nan::Get(sp, Nan::New<String>("hours").ToLocalChecked()).ToLocalChecked()).ToLocalChecked()->Value(),
						Nan::To<Number>(Nan::Get(sp, Nan::New<String>("minutes").ToLocalChecked()).ToLocalChecked()).ToLocalChecked()->Value(),
						Nan::To<Number>(Nan::Get(sp, Nan::New<String>("setback").ToLocalChecked()).ToLocalChecked()).ToLocalChecked()->Value()
					);
				}
			}
		}
	}

	// =================================================================
	NAN_METHOD(OZW::RemoveSwitchPoint)
	// =================================================================
	{
		Nan::HandleScope scope;
		CheckMinArgs(1, "valueId");
		OpenZWave::ValueID* vit = populateValueId(info);
		if (vit) {
			if ((*vit).GetType() != OpenZWave::ValueID::ValueType_Schedule ) {
				Nan::ThrowTypeError("OpenZWave valueId is not a ValueType_Schedule");
			} else {
				uint8 idxpos  =  (info[0]->IsObject()) ? 1 : 4;
				if ((info.Length() < idxpos) || !info[idxpos]->IsObject()) {
					Nan::ThrowTypeError("must supply a switchpoint object ");
				} else {
					Local<Object> sp = Nan::To<Object>(info[idxpos]).ToLocalChecked();
					OpenZWave::Manager::Get()->RemoveSwitchPoint(*vit,
						Nan::To<Number>(Nan::Get(sp, Nan::New<String>("hours").ToLocalChecked()).ToLocalChecked()).ToLocalChecked()->Value(),
						Nan::To<Number>(Nan::Get(sp, Nan::New<String>("minutes").ToLocalChecked()).ToLocalChecked()).ToLocalChecked()->Value()
					);
				}
			}
		}
	}

}
